@@ -135,6 +135,7 @@ struct options_t {
   struct in_addr uamlogout;      /* IP address of HTTP auto-logout */
   struct in_addr uamalias;       /* IP address of UAM Alias */
   char *uamaliasname;            /* Simple hostname (no dots) DNS name for uamalias */
+  char *uamhostname;             /* Simple hostname (no dots) DNS name for uamlisten */
 
   /* booleans */
   uint8_t allowdyn:1;               /* Allow dynamic address allocation */
@@ -172,6 +173,7 @@ struct options_t {
   uint8_t redirssl:1;               /* Enable redirection of SSL/HTTPS port (requires SSL support) */
   uint8_t uamuissl:1;               /* Enable SSL/HTTPS on uamuiport (requires SSL support) */
   uint8_t domaindnslocal:1;         /* Wildcard option to consider all hostnames *.domain local */
+  uint8_t radsec:1;                 /* Use RadSec tunneling */
   /* */
 
   pass_through pass_throughs[MAX_PASS_THROUGHS];
@@ -201,7 +203,9 @@ struct options_t {
 
 #ifdef HAVE_SSL
   char *sslkeyfile;
+  char *sslkeypass;
   char *sslcertfile;
+  char *sslcafile;
 #endif
 
   /* local content */
