@@ -374,6 +374,21 @@ static int bstring_buildurl(bstring str, struct redir_conn_t *conn,
     bconcat(str, bt2);
   }
 
+  if (_options.uamuissl && _options.uamuiport) {
+    /*
+     *  When we have uamuissl, a key/cert, and a uamuiport,
+     *  then let's inform the captive portal of an SSL enabled
+     *  services. 
+     */
+    bcatcstr(str, amp);
+    bcatcstr(str, "ssl=");
+    bassignformat(bt, "https://%s:%d/", 
+		  inet_ntoa(_options.uamalias),
+		  _options.uamuiport);
+    redir_urlencode(bt, bt2);
+    bconcat(str, bt2);
+  }
+
   if (redirurl) {
     bcatcstr(str, amp);
     bcatcstr(str, "redirurl=");
@@ -390,7 +405,8 @@ static int bstring_buildurl(bstring str, struct redir_conn_t *conn,
     bconcat(str, bt2);
   }
 
-  if (redir->secret && *redir->secret) { /* take the md5 of the url+uamsecret as a checksum */
+  if (redir->secret && *redir->secret) { 
+    /* take the md5 of the url+uamsecret as a checksum */
     redir_md_param(str, redir->secret, amp);
   }
 
@@ -884,7 +900,6 @@ static int redir_json_reply(struct redir_t *redir, int res, struct redir_conn_t
     bcatcstr(json, ")");
   }
 
-
   redir_http(s, "200 OK");
 
   bcatcstr(s, "Content-Length: ");
@@ -1070,14 +1085,14 @@ int redir_listen(struct redir_t *redir) {
 
   for (n = 0; n < 2 && redir->fd[n]; n++) {
 
+    address.sin_addr.s_addr = redir->addr.s_addr;
     switch(n) {
     case 0:
-      address.sin_addr.s_addr = redir->addr.s_addr;
       address.sin_port = htons(redir->port);
       break;
     case 1:
       /* XXX: binding to 0.0.0.0:uiport (should be configurable?) */
-      address.sin_addr.s_addr = INADDR_ANY;
+      /*address.sin_addr.s_addr = INADDR_ANY;*/
       address.sin_port = htons(redir->uiport);
       break;
     }
@@ -1369,7 +1384,7 @@ static int redir_getreq(struct redir_t *redir, struct redir_socket_t *sock,
       if (sock->sslcon) {
 	recvlen = openssl_read(sock->sslcon, 
 			       buffer + buflen, 
-			       httpreq->allow_post ? 1 : sizeof(buffer) - 1 - buflen, 
+			       httpreq->allow_post ? 1 : (sizeof(buffer) - 1 - buflen), 
 			       0);
       } else
 #endif
@@ -1486,7 +1501,7 @@ static int redir_getreq(struct redir_t *redir, struct redir_socket_t *sock,
 	}
       } else if (linelen == 0) { 
 	/* end of headers */
-	/*log_dbg("end of http-request");*/
+	log_dbg("end of http-request");
 	done = 1;
 	break;
       } else { 
@@ -2335,10 +2350,20 @@ pid_t redir_fork(int in, int out) {
 int redir_main_exit(struct redir_t *redir, struct redir_httpreq_t *httpreq, 
 		    struct redir_socket_t *socket, int forked) {
   if (httpreq->data_in) bdestroy(httpreq->data_in);
+#ifdef HAVE_SSL
+  if (socket->sslcon) {
+    openssl_shutdown(socket->sslcon, 2);
+    openssl_free(socket->sslcon);
+    socket->sslcon=0;
+  }
+#endif
   if (forked) _redir_close_exit(socket->fd[0], socket->fd[1]);
   return _redir_close(socket->fd[0], socket->fd[1]);
 }
 
+static int redir_signal2(int sig) {
+  log_dbg("signal(%d)=%d", getpid(),sig);
+}
 static int redir_close_on_signal[2]={0,0};
 static int redir_signal(int sig) {
   close(redir_close_on_signal[0]);
@@ -2474,7 +2499,7 @@ int redir_main(struct redir_t *redir,
     return redir_main_exit(redir, &httpreq, &socket, forked);
   }
 
-  log_dbg("Process HTTP Request");
+  log_dbg("Processing HTTP%s Request", (conn.flags & USING_SSL) ? "S" : "");
   
   if (conn.type == REDIR_WWW) {
     pid_t forkpid;
@@ -2559,7 +2584,7 @@ int redir_main(struct redir_t *redir,
 
 	  forkpid = redir_fork(ptoc[0], ctop[1]);
 
-	  act.sa_handler = redir_signal;
+	  act.sa_handler = redir_signal2;
 	  sigaction(SIGCHLD, &act, NULL);
 	  
 	  act.sa_handler = redir_signal;
@@ -2571,10 +2596,10 @@ int redir_main(struct redir_t *redir,
 	  }
 
 	  forked = 1;
-	  if (forkpid) { 
+	  if (forkpid > 0) { 
 	    /* parent */
 
-	    int clen = httpreq.clen;
+	    int rd, clen = httpreq.clen;
 
 	    close(ptoc[0]);
 	    close(ctop[1]);
@@ -2585,7 +2610,9 @@ int redir_main(struct redir_t *redir,
 	    log_dbg("ssl_wrapper(%d)", getpid());
 
 	    while (clen > 0) {
-	      if ((buflen = openssl_read(socket.sslcon, buffer, bufsize, 1)) > 0) {
+	      rd = clen > bufsize ? bufsize : clen;
+	      log_dbg("reading(%d)", rd);
+	      if ((buflen = openssl_read(socket.sslcon, buffer, rd, 0)) > 0) {
 		if (write(ptoc[1], buffer, (size_t) buflen) < 0) {
 		  log_err(errno, "tcp_write() failed!");
 		  return redir_main_exit(redir, &httpreq, &socket, forked);
@@ -2608,6 +2635,8 @@ int redir_main(struct redir_t *redir,
 		break;
 	      }
 	    }
+
+	    log_dbg("ssl_wrapper(%d) done", getpid());
 	    
 	    close(ptoc[1]);
  	    close(ctop[0]);
@@ -2617,6 +2646,7 @@ int redir_main(struct redir_t *redir,
 
 	  close(ptoc[1]);
 	  close(ctop[0]);
+
 	  /* child */
 	  log_dbg("script(%d)", getpid());
 	}
@@ -2630,6 +2660,7 @@ int redir_main(struct redir_t *redir,
 #ifdef HAVE_SSL
 	  if (socket.sslcon) {
 	    setenv("HTTPS", "on", 1);
+	    sleep(1);
 	  }
 #endif
 
